@@ -26,7 +26,9 @@
 
 #include <rfb/ComparingUpdateTracker.h>
 #include <rfb/Encoder.h>
+#include <rfb/Exception.h>
 #include <rfb/KeyRemapper.h>
+#include <rfb/KeysymStr.h>
 #include <rfb/LogWriter.h>
 #include <rfb/Security.h>
 #include <rfb/ServerCore.h>
@@ -40,6 +42,7 @@
 #define XK_MISCELLANY
 #define XK_XKB_KEYS
 #include <rfb/keysymdef.h>
+#include <rfb/util.h>
 
 using namespace rfb;
 
@@ -48,8 +51,9 @@ static LogWriter vlog("VNCSConnST");
 static Cursor emptyCursor(0, 0, Point(0, 0), NULL);
 
 VNCSConnectionST::VNCSConnectionST(VNCServerST* server_, network::Socket *s,
-                                   bool reverse)
-  : sock(s), reverseConnection(reverse),
+                                   bool reverse, AccessRights ar)
+  : SConnection(ar),
+    sock(s), reverseConnection(reverse),
     inProcessMessages(false),
     pendingSyncFence(false), syncFence(false), fenceFlags(0),
     fenceDataLen(0), fenceData(NULL), congestionTimer(this),
@@ -59,7 +63,7 @@ VNCSConnectionST::VNCSConnectionST(VNCServerST* server_, network::Socket *s,
     pointerEventTime(0), clientHasCursor(false)
 {
   setStreams(&sock->inStream(), &sock->outStream());
-  peerEndpoint.buf = sock->getPeerEndpoint();
+  peerEndpoint = sock->getPeerEndpoint();
 
   // Kick off the idle timer
   if (rfb::Server::idleTimeout) {
@@ -75,19 +79,23 @@ VNCSConnectionST::VNCSConnectionST(VNCServerST* server_, network::Socket *s,
 VNCSConnectionST::~VNCSConnectionST()
 {
   // If we reach here then VNCServerST is deleting us!
-  if (closeReason.buf)
-    vlog.info("closing %s: %s", peerEndpoint.buf, closeReason.buf);
+  if (!closeReason.empty())
+    vlog.info("closing %s: %s", peerEndpoint.c_str(),
+              closeReason.c_str());
+
+  // Release any mouse buttons
+  server->pointerEvent(this, server->getCursorPos(), 0);
 
   // Release any keys the client still had pressed
   while (!pressedKeys.empty()) {
-    rdr::U32 keysym, keycode;
+    uint32_t keysym, keycode;
 
     keysym = pressedKeys.begin()->second;
     keycode = pressedKeys.begin()->first;
     pressedKeys.erase(pressedKeys.begin());
 
-    vlog.debug("Releasing key 0x%x / 0x%x on client disconnect",
-               keysym, keycode);
+    vlog.debug("Releasing key 0x%04x / XK_%s (0x%04x) on client disconnect",
+               keycode, KeySymName(keysym), keysym);
     server->keyEvent(keysym, keycode, false);
   }
 
@@ -112,10 +120,10 @@ void VNCSConnectionST::close(const char* reason)
   SConnection::close(reason);
 
   // Log the reason for the close
-  if (!closeReason.buf)
-    closeReason.buf = strDup(reason);
+  if (closeReason.empty())
+    closeReason = reason;
   else
-    vlog.debug("second close: %s (%s)", peerEndpoint.buf, reason);
+    vlog.debug("second close: %s (%s)", peerEndpoint.c_str(), reason);
 
   try {
     if (sock->outStream().hasBufferedData()) {
@@ -261,7 +269,7 @@ void VNCSConnectionST::writeFramebufferUpdateOrClose()
   }
 }
 
-void VNCSConnectionST::screenLayoutChangeOrClose(rdr::U16 reason)
+void VNCSConnectionST::screenLayoutChangeOrClose(uint16_t reason)
 {
   try {
     screenLayoutChange(reason);
@@ -313,9 +321,9 @@ void VNCSConnectionST::setLEDStateOrClose(unsigned int state)
 void VNCSConnectionST::requestClipboardOrClose()
 {
   try {
+    if (state() != RFBSTATE_NORMAL) return;
     if (!accessCheck(AccessCutText)) return;
     if (!rfb::Server::acceptCutText) return;
-    if (state() != RFBSTATE_NORMAL) return;
     requestClipboard();
   } catch(rdr::Exception& e) {
     close(e.str());
@@ -325,9 +333,9 @@ void VNCSConnectionST::requestClipboardOrClose()
 void VNCSConnectionST::announceClipboardOrClose(bool available)
 {
   try {
+    if (state() != RFBSTATE_NORMAL) return;
     if (!accessCheck(AccessCutText)) return;
     if (!rfb::Server::sendCutText) return;
-    if (state() != RFBSTATE_NORMAL) return;
     announceClipboard(available);
   } catch(rdr::Exception& e) {
     close(e.str());
@@ -337,9 +345,9 @@ void VNCSConnectionST::announceClipboardOrClose(bool available)
 void VNCSConnectionST::sendClipboardDataOrClose(const char* data)
 {
   try {
+    if (state() != RFBSTATE_NORMAL) return;
     if (!accessCheck(AccessCutText)) return;
     if (!rfb::Server::sendCutText) return;
-    if (state() != RFBSTATE_NORMAL) return;
     sendClipboardData(data);
   } catch(rdr::Exception& e) {
     close(e.str());
@@ -400,7 +408,7 @@ bool VNCSConnectionST::needRenderedCursor()
 
   if (!client.supportsLocalCursor())
     return true;
-  if (!server->getCursorPos().equals(pointerEventPos) &&
+  if ((server->getCursorPos() != pointerEventPos) &&
       (time(0) - pointerEventTime) > 0)
     return true;
 
@@ -502,8 +510,8 @@ public:
 
 // keyEvent() - record in the pressedKeys which keys were pressed.  Allow
 // multiple down events (for autorepeat), but only allow a single up event.
-void VNCSConnectionST::keyEvent(rdr::U32 keysym, rdr::U32 keycode, bool down) {
-  rdr::U32 lookup;
+void VNCSConnectionST::keyEvent(uint32_t keysym, uint32_t keycode, bool down) {
+  uint32_t lookup;
 
   if (rfb::Server::idleTimeout)
     idleTimer.start(secsToMillis(rfb::Server::idleTimeout));
@@ -511,9 +519,11 @@ void VNCSConnectionST::keyEvent(rdr::U32 keysym, rdr::U32 keycode, bool down) {
   if (!rfb::Server::acceptKeyEvents) return;
 
   if (down)
-    vlog.debug("Key pressed: 0x%x / 0x%x", keysym, keycode);
+    vlog.debug("Key pressed: 0x%04x / XK_%s (0x%04x)",
+               keycode, KeySymName(keysym), keysym);
   else
-    vlog.debug("Key released: 0x%x / 0x%x", keysym, keycode);
+    vlog.debug("Key released: 0x%04x / XK_%s (0x%04x)",
+               keycode, KeySymName(keysym), keysym);
 
   // Avoid lock keys if we don't know the server state
   if ((server->getLEDState() == ledUnknown) &&
@@ -669,9 +679,9 @@ void VNCSConnectionST::setDesktopSize(int fb_width, int fb_height,
   writer()->writeDesktopSize(reasonClient, result);
 }
 
-void VNCSConnectionST::fence(rdr::U32 flags, unsigned len, const char data[])
+void VNCSConnectionST::fence(uint32_t flags, unsigned len, const uint8_t data[])
 {
-  rdr::U8 type;
+  uint8_t type;
 
   if (flags & fenceFlagRequest) {
     if (flags & fenceFlagSyncNext) {
@@ -682,7 +692,7 @@ void VNCSConnectionST::fence(rdr::U32 flags, unsigned len, const char data[])
       delete [] fenceData;
       fenceData = NULL;
       if (len > 0) {
-        fenceData = new char[len];
+        fenceData = new uint8_t[len];
         memcpy(fenceData, data, len);
       }
 
@@ -768,7 +778,7 @@ void VNCSConnectionST::supportsLocalCursor()
 
 void VNCSConnectionST::supportsFence()
 {
-  char type = 0;
+  uint8_t type = 0;
   writer()->writeFence(fenceFlagRequest, sizeof(type), &type);
 }
 
@@ -790,7 +800,7 @@ void VNCSConnectionST::supportsLEDState()
   writer()->writeLEDState();
 }
 
-bool VNCSConnectionST::handleTimeout(Timer* t)
+void VNCSConnectionST::handleTimeout(Timer* t)
 {
   try {
     if ((t == &congestionTimer) ||
@@ -802,13 +812,11 @@ bool VNCSConnectionST::handleTimeout(Timer* t)
 
   if (t == &idleTimer)
     close("Idle timeout");
-
-  return false;
 }
 
 bool VNCSConnectionST::isShiftPressed()
 {
-    std::map<rdr::U32, rdr::U32>::const_iterator iter;
+    std::map<uint32_t, uint32_t>::const_iterator iter;
 
     for (iter = pressedKeys.begin(); iter != pressedKeys.end(); ++iter) {
       if (iter->second == XK_Shift_L)
@@ -822,7 +830,7 @@ bool VNCSConnectionST::isShiftPressed()
 
 void VNCSConnectionST::writeRTTPing()
 {
-  char type;
+  uint8_t type;
 
   if (!client.supportsFence())
     return;
@@ -1108,7 +1116,7 @@ void VNCSConnectionST::writeLosslessRefresh()
 }
 
 
-void VNCSConnectionST::screenLayoutChange(rdr::U16 reason)
+void VNCSConnectionST::screenLayoutChange(uint16_t reason)
 {
   if (!authenticated())
     return;
