@@ -26,7 +26,10 @@
 #include <signal.h>
 #include <unistd.h>
 
+#include <network/Socket.h>
+
 #include <rfb/LogWriter.h>
+#include <rfb/Exception.h>
 
 #include <x0vncserver/XDesktop.h>
 
@@ -51,7 +54,6 @@ void vncSetGlueContext(Display *dpy, void *res);
 #include <x0vncserver/Geometry.h>
 #include <x0vncserver/XPixelBuffer.h>
 
-using namespace std;
 using namespace rfb;
 
 extern const unsigned short code_map_qnum_to_xorgevdev[];
@@ -230,9 +232,13 @@ void XDesktop::poll() {
   }
 }
 
+void XDesktop::init(VNCServer* vs)
+{
+  server = vs;
+}
 
-void XDesktop::start(VNCServer* vs) {
-
+void XDesktop::start()
+{
   // Determine actual number of buttons of the X pointer device.
   unsigned char btnMap[8];
   int numButtons = XGetPointerMapping(dpy, btnMap, 8);
@@ -247,7 +253,6 @@ void XDesktop::start(VNCServer* vs) {
   pb = new XPixelBuffer(dpy, factory, geometry->getRect());
   vlog.info("Allocated %s", pb->getImage()->classDesc());
 
-  server = vs;
   server->setPixelBuffer(pb, computeScreenLayout());
 
 #ifdef HAVE_XDAMAGE
@@ -276,8 +281,10 @@ void XDesktop::start(VNCServer* vs) {
 void XDesktop::stop() {
   running = false;
 
+#ifdef HAVE_XTEST
   // Delete added keycodes
   deleteAddedKeysyms(dpy);
+#endif
 
 #ifdef HAVE_XDAMAGE
   if (haveDamage)
@@ -288,7 +295,6 @@ void XDesktop::stop() {
   queryConnectDialog = 0;
 
   server->setPixelBuffer(0);
-  server = 0;
 
   delete pb;
   pb = 0;
@@ -327,9 +333,9 @@ void XDesktop::queryConnection(network::Socket* sock,
 
   queryConnectSock = sock;
 
-  CharArray address(sock->getPeerAddress());
   delete queryConnectDialog;
-  queryConnectDialog = new QueryConnectDialog(dpy, address.buf,
+  queryConnectDialog = new QueryConnectDialog(dpy,
+                                              sock->getPeerAddress(),
                                               userName,
                                               queryConnectTimeout,
                                               this);
@@ -355,6 +361,9 @@ void XDesktop::pointerEvent(const Point& pos, int buttonMask) {
     }
   }
   oldButtonMask = buttonMask;
+#else
+  (void)pos;
+  (void)buttonMask;
 #endif
 }
 
@@ -396,7 +405,6 @@ KeyCode XDesktop::XkbKeysymToKeycode(Display* dpy, KeySym keysym) {
 
   return keycode;
 }
-#endif
 
 KeyCode XDesktop::addKeysym(Display* dpy, KeySym keysym)
 {
@@ -481,6 +489,10 @@ void XDesktop::deleteAddedKeysyms(Display* dpy) {
     }
   }
 
+  // Did we actually find something to remove?
+  if (highestKeyCode < lowestKeyCode)
+    return;
+
   changes.changed |= XkbKeySymsMask;
   changes.first_key_sym = lowestKeyCode;
   changes.num_key_syms = highestKeyCode - lowestKeyCode + 1;
@@ -508,9 +520,10 @@ KeyCode XDesktop::keysymToKeycode(Display* dpy, KeySym keysym) {
 
   return keycode;
 }
+#endif
 
 
-void XDesktop::keyEvent(rdr::U32 keysym, rdr::U32 xtcode, bool down) {
+void XDesktop::keyEvent(uint32_t keysym, uint32_t xtcode, bool down) {
 #ifdef HAVE_XTEST
   int keycode = 0;
 
@@ -542,10 +555,14 @@ void XDesktop::keyEvent(rdr::U32 keysym, rdr::U32 xtcode, bool down) {
   vlog.debug("%d %s", keycode, down ? "down" : "up");
 
   XTestFakeKeyEvent(dpy, keycode, down, CurrentTime);
+#else
+  (void)keysym;
+  (void)xtcode;
+  (void)down;
 #endif
 }
 
-void XDesktop::clientCutText(const char* str) {
+void XDesktop::clientCutText(const char* /*str*/) {
 }
 
 ScreenSet XDesktop::computeScreenLayout()
@@ -774,6 +791,9 @@ unsigned int XDesktop::setScreenLayout(int fb_width, int fb_height,
   return ret;
 
 #else
+  (void)fb_width;
+  (void)fb_height;
+  (void)layout;
   return rfb::resultProhibited;
 #endif /* HAVE_XRANDR */
 }
@@ -934,6 +954,7 @@ void XDesktop::queryRejected()
   queryConnectSock = 0;
 }
 
+#ifdef HAVE_XFIXES
 bool XDesktop::setCursor()
 {
   XFixesCursorImage *cim;
@@ -944,20 +965,20 @@ bool XDesktop::setCursor()
 
   // Copied from XserverDesktop::setCursor() in
   // unix/xserver/hw/vnc/XserverDesktop.cc and adapted to
-  // handle long -> U32 conversion for 64-bit Xlib
-  rdr::U8* cursorData;
-  rdr::U8 *out;
+  // handle long -> uint32_t conversion for 64-bit Xlib
+  uint8_t* cursorData;
+  uint8_t *out;
   const unsigned long *pixels;
 
-  cursorData = new rdr::U8[cim->width * cim->height * 4];
+  cursorData = new uint8_t[cim->width * cim->height * 4];
 
   // Un-premultiply alpha
   pixels = cim->pixels;
   out = cursorData;
   for (int y = 0; y < cim->height; y++) {
     for (int x = 0; x < cim->width; x++) {
-      rdr::U8 alpha;
-      rdr::U32 pixel = *pixels++;
+      uint8_t alpha;
+      uint32_t pixel = *pixels++;
 
       alpha = (pixel >> 24) & 0xff;
       if (alpha == 0)
@@ -981,3 +1002,4 @@ bool XDesktop::setCursor()
   XFree(cim);
   return true;
 }
+#endif
