@@ -28,7 +28,9 @@
 #include <rfb/CMsgWriter.h>
 #include <rfb/LogWriter.h>
 #include <rfb/Exception.h>
+#include <rfb/KeysymStr.h>
 #include <rfb/ledStates.h>
+#include <rfb/util.h>
 
 // FLTK can pull in the X11 headers on some systems
 #ifndef XK_VoidSymbol
@@ -55,12 +57,13 @@
 #define MAPVK_VK_TO_VSC 0
 #endif
 
+#include "fltk/layout.h"
+#include "fltk/util.h"
 #include "Viewport.h"
 #include "CConn.h"
 #include "OptionsDialog.h"
 #include "DesktopWindow.h"
 #include "i18n.h"
-#include "fltk_layout.h"
 #include "parameters.h"
 #include "keysym2ucs.h"
 #include "menukey.h"
@@ -73,6 +76,7 @@
 
 #include <FL/Fl_Menu.H>
 #include <FL/Fl_Menu_Button.H>
+#include <FL/x.H>
 
 #if !defined(WIN32) && !defined(__APPLE__)
 #include <X11/XKBlib.h>
@@ -97,7 +101,6 @@ extern const unsigned int code_map_osx_to_qnum_len;
 
 
 using namespace rfb;
-using namespace rdr;
 
 static rfb::LogWriter vlog("Viewport");
 
@@ -112,14 +115,13 @@ enum { ID_DISCONNECT, ID_FULLSCREEN, ID_MINIMIZE, ID_RESIZE,
 static const WORD SCAN_FAKE = 0xaa;
 #endif
 
-Viewport::Viewport(int w, int h, const rfb::PixelFormat& serverPF, CConn* cc_)
+Viewport::Viewport(int w, int h, const rfb::PixelFormat& /*serverPF*/, CConn* cc_)
   : Fl_Widget(0, 0, w, h), cc(cc_), frameBuffer(NULL),
     lastPointerPos(0, 0), lastButtonMask(0),
 #ifdef WIN32
     altGrArmed(false),
 #endif
-    firstLEDState(true),
-    pendingServerClipboard(false), pendingClientClipboard(false),
+    firstLEDState(true), pendingClientClipboard(false),
     menuCtrlKey(false), menuAltKey(false), cursor(NULL)
 {
 #if !defined(WIN32) && !defined(__APPLE__)
@@ -244,7 +246,7 @@ static const char * dotcursor_xpm[] = {
   "     "};
 
 void Viewport::setCursor(int width, int height, const Point& hotspot,
-                         const rdr::U8* data)
+                         const uint8_t* data)
 {
   int i;
 
@@ -265,12 +267,12 @@ void Viewport::setCursor(int width, int height, const Point& hotspot,
     cursorHotspot.x = cursorHotspot.y = 2;
   } else {
     if ((width == 0) || (height == 0)) {
-      U8 *buffer = new U8[4];
+      uint8_t *buffer = new uint8_t[4];
       memset(buffer, 0, 4);
       cursor = new Fl_RGB_Image(buffer, 1, 1, 4);
       cursorHotspot.x = cursorHotspot.y = 0;
     } else {
-      U8 *buffer = new U8[width * height * 4];
+      uint8_t *buffer = new uint8_t[width * height * 4];
       memcpy(buffer, data, width * height * 4);
       cursor = new Fl_RGB_Image(buffer, width, height, 4);
       cursorHotspot = hotspot;
@@ -293,18 +295,16 @@ void Viewport::handleClipboardAnnounce(bool available)
 
   if (!available) {
     vlog.debug("Clipboard is no longer available on server");
-    pendingServerClipboard = false;
+    return;
+  }
+
+  if (!hasFocus()) {
+    vlog.debug("Got notification of new clipboard on server whilst not focused, ignoring");
     return;
   }
 
   pendingClientClipboard = false;
 
-  if (!hasFocus()) {
-    vlog.debug("Got notification of new clipboard on server whilst not focused, will request data later");
-    pendingServerClipboard = true;
-    return;
-  }
-
   vlog.debug("Got notification of new clipboard on server, requesting data");
   cc->requestClipboard();
 }
@@ -558,26 +558,27 @@ void Viewport::resize(int x, int y, int w, int h)
 
 int Viewport::handle(int event)
 {
-  char *filtered;
+  std::string filtered;
   int buttonMask, wheelMask;
   DownMap::const_iterator iter;
 
   switch (event) {
   case FL_PASTE:
-    filtered = convertLF(Fl::event_text(), Fl::event_length());
-
-    vlog.debug("Sending clipboard data (%d bytes)", (int)strlen(filtered));
-
-    try {
-      cc->sendClipboardData(filtered);
-    } catch (rdr::Exception& e) {
-      vlog.error("%s", e.str());
-      abort_connection(_("An unexpected error occurred when "
-                         "communicating with the server:\n\n%s"),
-                       e.str());
+    if (!isValidUTF8(Fl::event_text(), Fl::event_length())) {
+      vlog.error("Invalid UTF-8 sequence in system clipboard");
+      return 1;
     }
 
-    strFree(filtered);
+    filtered = convertLF(Fl::event_text(), Fl::event_length());
+
+    vlog.debug("Sending clipboard data (%d bytes)", (int)filtered.size());
+
+    try {
+      cc->sendClipboardData(filtered.c_str());
+    } catch (rdr::Exception& e) {
+      vlog.error("%s", e.str());
+      abort_connection_with_unexpected_error(e);
+    }
 
     return 1;
 
@@ -644,10 +645,9 @@ int Viewport::handle(int event)
     return 1;
 
   case FL_UNFOCUS:
-    // Release all keys that were pressed as that generally makes most
-    // sense (e.g. Alt+Tab where we only see the Alt press)
-    while (!downKeySym.empty())
-      handleKeyRelease(downKeySym.begin()->first);
+    // We won't get more key events, so reset our knowledge about keys
+    resetKeyboard();
+
     Fl::enable_im();
     return 1;
 
@@ -670,9 +670,7 @@ void Viewport::sendPointerEvent(const rfb::Point& pos, int buttonMask)
       cc->writer()->writePointerEvent(pos, buttonMask);
     } catch (rdr::Exception& e) {
       vlog.error("%s", e.str());
-      abort_connection(_("An unexpected error occurred when "
-                         "communicating with the server:\n\n%s"),
-                       e.str());
+      abort_connection_with_unexpected_error(e);
     }
   } else {
     if (!Fl::has_timeout(handlePointerTimeout, this))
@@ -758,8 +756,6 @@ void Viewport::handleClipboardChange(int source, void *data)
 
   self->clipboardSource = source;
 
-  self->pendingServerClipboard = false;
-
   if (!self->hasFocus()) {
     vlog.debug("Local clipboard changed whilst not focused, will notify server later");
     self->pendingClientClipboard = true;
@@ -773,39 +769,23 @@ void Viewport::handleClipboardChange(int source, void *data)
     self->cc->announceClipboard(true);
   } catch (rdr::Exception& e) {
     vlog.error("%s", e.str());
-    abort_connection(_("An unexpected error occurred when "
-                       "communicating with the server:\n\n%s"),
-                     e.str());
+    abort_connection_with_unexpected_error(e);
   }
 }
 
 
 void Viewport::flushPendingClipboard()
 {
-  if (pendingServerClipboard) {
-    vlog.debug("Focus regained after remote clipboard change, requesting data");
-    try {
-      cc->requestClipboard();
-    } catch (rdr::Exception& e) {
-      vlog.error("%s", e.str());
-      abort_connection(_("An unexpected error occurred when "
-                         "communicating with the server:\n\n%s"),
-                       e.str());
-    }
-  }
   if (pendingClientClipboard) {
     vlog.debug("Focus regained after local clipboard change, notifying server");
     try {
       cc->announceClipboard(true);
     } catch (rdr::Exception& e) {
       vlog.error("%s", e.str());
-      abort_connection(_("An unexpected error occurred when "
-                         "communicating with the server:\n\n%s"),
-                       e.str());
+      abort_connection_with_unexpected_error(e);
     }
   }
 
-  pendingServerClipboard = false;
   pendingClientClipboard = false;
 }
 
@@ -827,14 +807,19 @@ void Viewport::handlePointerTimeout(void *data)
                                           self->lastButtonMask);
   } catch (rdr::Exception& e) {
     vlog.error("%s", e.str());
-    abort_connection(_("An unexpected error occurred when "
-                       "communicating with the server:\n\n%s"),
-                     e.str());
+    abort_connection_with_unexpected_error(e);
   }
 }
 
 
-void Viewport::handleKeyPress(int keyCode, rdr::U32 keySym)
+void Viewport::resetKeyboard()
+{
+  while (!downKeySym.empty())
+    handleKeyRelease(downKeySym.begin()->first);
+}
+
+
+void Viewport::handleKeyPress(int keyCode, uint32_t keySym)
 {
   static bool menuRecursion = false;
 
@@ -883,12 +868,8 @@ void Viewport::handleKeyPress(int keyCode, rdr::U32 keySym)
   // and send the same on release.
   downKeySym[keyCode] = keySym;
 
-#if defined(WIN32) || defined(__APPLE__)
-  vlog.debug("Key pressed: 0x%04x => 0x%04x", keyCode, keySym);
-#else
   vlog.debug("Key pressed: 0x%04x => XK_%s (0x%04x)",
-             keyCode, XKeysymToString(keySym), keySym);
-#endif
+             keyCode, KeySymName(keySym), keySym);
 
   try {
     // Fake keycode?
@@ -898,9 +879,7 @@ void Viewport::handleKeyPress(int keyCode, rdr::U32 keySym)
       cc->writer()->writeKeyEvent(keySym, keyCode, true);
   } catch (rdr::Exception& e) {
     vlog.error("%s", e.str());
-    abort_connection(_("An unexpected error occurred when "
-                       "communicating with the server:\n\n%s"),
-                     e.str());
+    abort_connection_with_unexpected_error(e);
   }
 }
 
@@ -920,12 +899,8 @@ void Viewport::handleKeyRelease(int keyCode)
     return;
   }
 
-#if defined(WIN32) || defined(__APPLE__)
-  vlog.debug("Key released: 0x%04x => 0x%04x", keyCode, iter->second);
-#else
   vlog.debug("Key released: 0x%04x => XK_%s (0x%04x)",
-             keyCode, XKeysymToString(iter->second), iter->second);
-#endif
+             keyCode, KeySymName(iter->second), iter->second);
 
   try {
     if (keyCode > 0xff)
@@ -934,9 +909,7 @@ void Viewport::handleKeyRelease(int keyCode)
       cc->writer()->writeKeyEvent(iter->second, keyCode, false);
   } catch (rdr::Exception& e) {
     vlog.error("%s", e.str());
-    abort_connection(_("An unexpected error occurred when "
-                       "communicating with the server:\n\n%s"),
-                     e.str());
+    abort_connection_with_unexpected_error(e);
   }
 
   downKeySym.erase(iter);
@@ -976,7 +949,7 @@ int Viewport::handleSystemEvent(void *event, void *data)
     UINT vKey;
     bool isExtended;
     int keyCode;
-    rdr::U32 keySym;
+    uint32_t keySym;
 
     vKey = msg->wParam;
     isExtended = (msg->lParam & (1 << 24)) != 0;
@@ -1140,6 +1113,12 @@ int Viewport::handleSystemEvent(void *event, void *data)
     return 1;
   }
 #elif defined(__APPLE__)
+  // Special event that means we temporarily lost some input
+  if (cocoa_is_keyboard_sync(event)) {
+    self->resetKeyboard();
+    return 1;
+  }
+
   if (cocoa_is_keyboard_event(event)) {
     int keyCode;
 
@@ -1150,7 +1129,7 @@ int Viewport::handleSystemEvent(void *event, void *data)
       keyCode = code_map_osx_to_qnum[keyCode];
 
     if (cocoa_is_key_press(event)) {
-      rdr::U32 keySym;
+      uint32_t keySym;
 
       keySym = cocoa_event_keysym(event);
       if (keySym == NoSymbol) {
@@ -1247,7 +1226,7 @@ void Viewport::initContextMenu()
 {
   contextMenu->clear();
 
-  fltk_menu_add(contextMenu, p_("ContextMenu|", "Dis&connect"),
+  fltk_menu_add(contextMenu, p_("ContextMenu|", "Disconn&ect"),
                 0, NULL, (void*)ID_DISCONNECT, FL_MENU_DIVIDER);
 
   fltk_menu_add(contextMenu, p_("ContextMenu|", "&Full screen"),
